% Sample code for
%   2D DOA estimation with sparse arrays in presence of mutual coupling
%
%	Written by Chun-Lin Liu
%	E-mail : cl.liu@caltech.edu
%	Project website:
%	https://systems.caltech.edu/dsp/students/clliu/hourglass.html
%	Last revised on July 31, 2017
%
%   If you use this script, please cite reference [1] and [2].
%
%	Reference
%   [1] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of the 50th Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
%   [2] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” IEEE Transactions on Signal Processing, vol. 65, no. 13, pp. 3369-3383, July 2017.
%


clear; clc; close all;

TYPES = 7;
name = cell(TYPES, 1);

%% ========== Sources ==========
DD = 36; % Number of sources
% Create source normalized DOAs
sqrt_DD = sqrt(DD);
mu_bar = [repmat(linspace(-0.3, 0.3, sqrt_DD).', [sqrt_DD, 1]), kron(linspace(-0.3, 0.3, sqrt_DD).', ones(sqrt_DD, 1))];
tt = 30; U = [cosd(tt), sind(tt); -sind(tt), cosd(tt)];
mu_bar = mu_bar * U;

% 2D normalized DOA (theta_bar, phi_bar)
theta_bar = mu_bar(:, 1);
phi_bar = mu_bar(:, 2);


% SNR
SNRdB = 0;
% Number of snapshots
SNAPSHOTS = 500;

% Mutual coupling parameters
c1 = 0.25;
B = 5;

for type = 1 : TYPES
    
    %% ========== Array geometries ==========
    switch (type)
        case 1
            name{type} = 'URA';
            Nx = 9; Ny = 9;
            S = uniform_rectangular_arrays( Nx, Ny );
        case 2
            name{type} = 'Billboard';
            Nx = 28; Ny = 28;
            S = billboard( Nx, Ny );
        case 3
            name{type} = 'Nested';
            N1 = 4; N2 = 5;
            S = [repmat( super_nested(N1, N2, 1), [N1+N2, 1] ), kron(super_nested(N1, N2, 1), ones(N1+N2, 1))];
        case 4
            name{type} = 'OBA';
            Nx = 29; Ny = 27;
            S = POBA_L( Nx, Ny, 'OBA' );
        case 5
            name{type} = 'HOBA';
            Nx = 29; Ny = 27;
            S = POBA_L( Nx, Ny, 'HOBA' );
        case 6
            name{type} = 'HOBA with 2 layers';
            Nx = 29; Ny = 27;
            S = POBA_L( Nx, Ny, 'HOBA-2' );
        case 7
            name{type} = 'Hourglass';
            Nx = 29; Ny = 27;
            S = POBA_L( Nx, Ny, 'hourglass' );
        otherwise
    end
    disp(name{type})
    
    % Number of sensors
    LEN_S = size(S, 1);
    
    tic;
    
    %% ========== Array output x_S ==========
    
    % Mutual coupling matrix
    C = mutual_coupling_C( S, 'Toeplitz', [c1, B] );
    
    V_S = exp(2i * pi * S * mu_bar');
    
    % Source and Noise
    UnitSource = (randn(DD, SNAPSHOTS) + 1i * randn(DD, SNAPSHOTS)) / sqrt(2);
    noise_std = 10^(-SNRdB/20);
    UnitNoise = (randn(LEN_S, SNAPSHOTS) + 1i * randn(LEN_S, SNAPSHOTS)) / sqrt(2);
    
    x_S = C * V_S * UnitSource + noise_std * UnitNoise;
    
    %% ========== Construct sample autocorrelation function ==========
    [ D, x_D ] = x_S_to_x_D( S, x_S );
    
    %% ========== 2D unitary ESPRIT on x_D ==========
    [theta_bar_est, phi_bar_est] = coarray_2D_unitary_ESPRIT( D, x_D, DD );
    
    toc;
    
    %% ========== Plot the true/estimated DOAs ==========
    figure;
    plot(theta_bar, phi_bar, 'o', ...
        theta_bar_est, phi_bar_est, '.');
    title(name{type})
    axis([-0.5, 0.5, -0.5, 0.5])
    axis square
    xlabel('$\bar{\theta}$', 'interpreter', 'latex')
    ylabel('$\bar{\phi}$', 'interpreter', 'latex')
    leg = legend('True DOA', 'Estimated DOA');
    set(leg, 'interpreter', 'latex')
    
end

