function C = mutual_coupling_C( S, method, param )
%MUTUAL_COUPLING_C: return the mutual matrix C for a given array.
%
%	Written by Chun-Lin Liu
%	E-mail : cl.liu@caltech.edu
%	Project website:
%	https://systems.caltech.edu/dsp/students/clliu/hourglass.html
%	Last revised on July 31, 2017
%
%   If you use this script, please cite related references
%
%	Reference
%   [1] B. Friedlander and A. J. Weiss, “Direction finding in the presence of mutual coupling,” in IEEE Transactions on Antennas and Propagation, vol. 39, no. 3, pp. 273-284, Mar 1991.
%   [2] C. A. Balanis, Antenna Theory: Analysis and Design, 4th ed. John Wiley & Sons, 2016.
%   [3] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Linear Sparse Arrays with Reduced Mutual Coupling - Part I: Fundamentals,” IEEE Transactions on Signal Processing, vol. 64, no. 15, pp. 3997-4012, Aug. 2016. 
%	[4] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Linear Sparse Arrays with Reduced Mutual Coupling - Part II: High-Order Extensions,” IEEE Transactions on Signal Processing, vol. 64, no. 16, pp. 4203-4217, Aug. 2016. 
%	[5] C.-L. Liu and P. P. Vaidyanathan, “Super Nested Arrays: Sparse Arrays with Less Mutual Coupling than Nested Arrays,” in Proc. of 2016 IEEE International Conference on Acoustics Speech and Signal Processing (ICASSP 2016), pp. 2976-2980, Shanghai, China, Mar. 2016.
%	[6] C.-L. Liu and P. P. Vaidyanathan, “High Order Super Nested Arrays,” in Proc. of the Ninth IEEE Sensor Array and Multichannel Signal Processing Workshop (SAM 2016), Rio de Janeiro, Brazil, Jul. 2016.
%   [7] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of the 50th Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
%   [8] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” IEEE Transactions on Signal Processing, vol. 65, no. 13, pp. 3369-3383, July 2017.
%
%
%   Input:
%       1) S: the sensor location, can be 1-D or 2-D
%           Example 1: S = [1; 2; 5] => 1-D array whose sensors are located
%           on 1, 2, and 5.
%           Example 2: S = [1, 3; 2, 2; -1, 3]
%           => Sensor locations are (1, 3), (2, 2), (-1, 3).
%
%       2) method: Different mutual coupling models
%               'Toeplitz': The symmetric Toeplitz model [1]
%               'Dipole': King's formula [2]
%               By default, it is 'Toeplitz' with
%                           c1 = 0.2, B = inf.
%
%       3) param: Additional parameter for mutual coupling model
%               If method = 'Toeplitz' [1],
%                   param = [c1, B].
%                   c1 = first coupling coefficient
%                   B = bandwidth
%               If method = 'Dipole' [2],
%                   dipole length = param(1) * lambda / 2,
%                   Z_L = param(2), and
%                   Z_A = param(3).
%                   By default, param = [1, 50, 50].
%
%   Output:
%       1) C: the mutual coupling matrix

[LEN_S, DIM] = size(S);

switch (DIM)
    case 1
        [Sx, Sx_] = ndgrid(S);
        d_pq = abs(Sx - Sx_);
    case 2
        [Sx, Sx_] = ndgrid(S(:, 1)); [Sy, Sy_] = ndgrid(S(:, 2));
        Sx_diff_mat = Sx - Sx_;
        Sy_diff_mat = Sy - Sy_;
        d_pq = sqrt(Sx_diff_mat.^2 + Sy_diff_mat.^2);
    otherwise
        error('This function only supports dimension up to 2.')
end

if (nargin == 1)
    method = 'Toeplitz';
end

switch (method)
    case 'Toeplitz'
        if (nargin <= 2)
            c1 = 0.2;
            B = inf;
        else
            c1 = param(1);
            B = param(2);
        end
        C = c1 ./ d_pq .* exp(2i * pi / 8 * (d_pq-1) );
        C( d_pq == 0 ) = 1;
        C( d_pq > B ) = 0;
    case 'Dipole'
        % Mutual coupling parameters
        if (nargin == 2)
            l_dipole = 1;
            Z_L = 50;
            Z_A = 50;
        else
            l_dipole = param(1);
            Z_L = param(2);
            Z_A = param(3);
        end
        % Antenna parameters
        beta = pi;
        eta_0 = 119.9169832*pi;
        
        u0 = beta*(sqrt(d_pq.^2+l_dipole^2)-l_dipole);
        v0 = beta*(sqrt(d_pq.^2+l_dipole^2)+l_dipole);
        u1 = beta*(sqrt(d_pq.^2+0.25*l_dipole^2)-l_dipole/2);
        v1 = beta*(sqrt(d_pq.^2+0.25*l_dipole^2)+l_dipole/2);
        
        R_pq = eta_0/(4*pi) * ( sin(beta*l_dipole) * ( -sinint(u0) + sinint(v0) + 2*sinint(u1) - 2*sinint(v1) ) + ...
            cos(beta*l_dipole) * (  cosint(u0) + cosint(v0) - 2*cosint(u1) - 2*cosint(v1) + 2*cosint(beta*d_pq) ) - ...
            ( 2*cosint(u1) + 2*cosint(v1) - 4*cosint(beta*d_pq) ));
        X_pq = eta_0/(4*pi) * ( sin(beta*l_dipole) * ( -cosint(u0) + cosint(v0) + 2*cosint(u1) - 2*cosint(v1) ) + ...
            cos(beta*l_dipole) * ( -sinint(u0) - sinint(v0) + 2*sinint(u1) + 2*sinint(v1) - 2*sinint(beta*d_pq) ) - ...
            ( 2*sinint(u1) + 2*sinint(v1) - 4*sinint(beta*d_pq) ));
        Z_pq = R_pq + 1i * X_pq;
        Z_pq( d_pq == 0 ) = eta_0/(4*pi) * (0.5772 + log(2*beta*l_dipole) - cosint(2*beta*l_dipole)) + 30i * sinint(2*beta*l_dipole);
        C = (Z_A + Z_L)*eye(LEN_S)\(Z_pq + Z_L*eye(LEN_S));
    otherwise
        error('Invalid method!')
end

end

